SystemOrganization addCategory: #Actors!
SystemOrganization addCategory: #'Actors-Tests'!

TestCase subclass: #ActorTests
	instanceVariableNames: 'actor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actors-Tests'!

!ActorTests methodsFor: 'running' stamp: 'bs 2/22/2010 08:21:52.274'!
actorPort
	^ 10000! !

!ActorTests methodsFor: 'tests' stamp: 'mt 9/8/2011 10:29:13.317'!
expectedFailures

	^ #(test05ActorConnectionLost)! !

!ActorTests methodsFor: 'running' stamp: 'mt 9/8/2011 09:49:43.101'!
setUp

	super setUp.
	
	actor := ActorMock new.
	actor startOn: self actorPort.! !

!ActorTests methodsFor: 'running' stamp: 'mt 9/8/2011 09:49:51.193'!
tearDown

	actor stop.
	
	super tearDown.! !

!ActorTests methodsFor: 'tests' stamp: 'mt 9/8/2011 09:57:16.836'!
test01ActorMessaging

	| testActorId aTextMessage |
	aTextMessage := 'Hello World'.
	testActorId := ActorId localhostPort: self actorPort.
	testActorId remember: aTextMessage.
	
	self
		wait;
		assert: aTextMessage equals: actor receivedMessage.
		
	testActorId close.! !

!ActorTests methodsFor: 'tests' stamp: 'mt 9/8/2011 09:53:11.305'!
test02ActorStartStop

	self assert: actor isListening.
	actor stop.
	self assert: actor isListening not.
	actor startOn: self actorPort.
	self assert: actor isListening.! !

!ActorTests methodsFor: 'tests' stamp: 'mt 9/8/2011 10:06:33.579'!
test03ActorIdConnection

	| id |
	id := ActorId localhostPort: self actorPort.
	id remember: 'This day.'. "lazy connection"
	
	self
		wait;
		assert: id isConnected.
	
	id close.
	self assert: id isConnected not.! !

!ActorTests methodsFor: 'tests' stamp: 'mt 9/8/2011 10:02:00.661'!
test04ActorStopProtection

	| id |
	id := ActorId localhostPort: self actorPort.
	self assert: actor isListening.
	id stop. "replicated to actor"
	self
		wait;
		assert: actor isListening.! !

!ActorTests methodsFor: 'tests' stamp: 'mt 9/8/2011 10:09:23.624'!
test05ActorConnectionLost

	| id |
	id := ActorId localhostPort: self actorPort.
	id remember: 'This day.'. "lazy connection"
	
	self
		wait;
		assert: id isConnected.
	
	actor stop. "actor connection lost"
	
	self
		wait;
		assert: id isConnected not.! !

!ActorTests methodsFor: 'tests' stamp: 'mt 9/8/2011 10:19:57.621'!
test06BigMessage

	| msg id |
	msg := String new: 1000000 withAll: $x.
	id := ActorId localhostPort: self actorPort.
	id remember: msg.
	
	self
		wait;
		assert: msg equals: actor receivedMessage.! !

!ActorTests methodsFor: 'tests' stamp: 'mt 9/8/2011 10:35:05.909'!
test07Recipient

	| recipient id message |
	recipient := ValueHolder new.
	message := 'Sunny weather.'.
	actor recipient: recipient.
	id := ActorId localhostPort: self actorPort.
	id contents: message.
	
	self
		wait;
		assert: message equals: recipient contents.
	! !

!ActorTests methodsFor: 'tests' stamp: 'mt 9/8/2011 11:28:10.235'!
test08MultipleMessages

	| id messages |
	messages := #('Sunny weather.' 'Thunderstorm.' 'Rainy.').
	id := ActorId localhostPort: self actorPort.

	messages do: [:msg | id remember: msg].
	
	self
		wait;
		assert: messages last equals: actor receivedMessage.
	! !

!ActorTests methodsFor: 'running' stamp: 'mt 9/8/2011 09:56:26.681'!
wait

	self wait: 100.! !

!ActorTests methodsFor: 'running' stamp: 'mt 9/8/2011 09:56:20.361'!
wait: milliseconds

	(Delay forMilliseconds: milliseconds) wait.! !

!SocketStream methodsFor: '*actors' stamp: 'mt 9/8/2011 10:24:15.843'!
upToAll: aStringOrByteArray timeout: seconds
	"Answer a subcollection from the current access position to the occurrence (if any, but not inclusive) of aStringOrByteArray. An empty string is returned after the timeout."

	| index sz result searchedSoFar target startTime |
	"Deal with ascii vs. binary"
	self isBinary
		ifTrue: [target := aStringOrByteArray asByteArray]
		ifFalse: [target := aStringOrByteArray asString].

	sz := target size.
	"Look in the current inBuffer first"
	index := inBuffer
		indexOfSubCollection: target
		startingAt: lastRead - sz + 2.
	(index > 0 and: [(index + sz) <= inNextToWrite]) ifTrue: ["found it"
		result := self nextInBuffer: index - lastRead - 1.
		self skip: sz.
		^ result].

	startTime := Time now.
	[searchedSoFar :=  self inBufferSize.
	"Receive more data"
	self receiveData.
	(startTime to: Time now) duration seconds < seconds] whileTrue: [

		"Data begins at lastRead + 1, we add searchedSoFar as offset and 
		backs up sz - 1 so that we can catch any borderline hits."

		index := inBuffer
			indexOfSubCollection: target 
			startingAt: (lastRead + searchedSoFar - sz + 2 max: 1).
		(index > 0 and: [(index + sz) <= inNextToWrite]) ifTrue: ["found it"
			result := self nextInBuffer: index - lastRead - 1.
			self skip: sz.
			^ result]].

	^ ''! !

!SocketStream methodsFor: '*actors' stamp: 'mt 9/8/2011 10:27:29.243'!
upToToken: aStringOrByteArray
	
	^ [self upToAll: aStringOrByteArray timeout: 600]
		on: ConnectionTimedOut
		do: [Transcript cr; show: '[SocketStream] Receiving FAILED. Connection timed out.'. ^ '']
		on: ConnectionClosed
		do: [Transcript cr; show: '[SocketStream] Receiving FAILED. Connection closed.'. ^ '']! !

Object subclass: #Actor
	instanceVariableNames: 'listenSocket servingProcess recipient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actors'!

!Actor commentStamp: 'bs 2/22/2010 08:58:10.193' prior: 0!
An actor represents a process that can receive messages.
- This simple implementation uses standard Smalltalk processes, thus, it is not very economic compared to much better actor implementations.
- On startUp it listens on a socket for incoming messsages.
- An actor can receive all message the normal object can handle. However, in contrast to normal object message send, an actor client does not expect an response. Actor like messaging is only one-way communication

Sources:
- Erlang concurrency
- Actor programming model!

!Actor class methodsFor: 'constants' stamp: 'mt 9/8/2011 09:25:30.386'!
messageEndMark

	^ 'ActorsEOF'! !

!Actor class methodsFor: 'constants' stamp: 'mt 9/8/2011 10:04:17.963'!
protectedMessages

	^#(startOn: stop process: runLoop serve:)! !

!Actor methodsFor: 'testing' stamp: 'mt 9/8/2011 10:05:56.469'!
canProcess: aMessage

	^ (aMessage isKindOf: Message)
		and: [self recipient respondsTo: aMessage selector]
		and: [self recipient ~~ self
			or: [(self class protectedMessages includes: aMessage selector) not]]
	! !

!Actor methodsFor: 'testing' stamp: 'mt 9/8/2011 09:51:54.243'!
isListening

	^ listenSocket isValid! !

!Actor methodsFor: 'processing' stamp: 'mt 9/8/2011 10:27:53.376'!
process: aSocket
	| socketStream messageBytes message |
	[aSocket isValid and: [aSocket isConnected] ] 
		whileTrue: [
			aSocket waitForDataIfClosed: [ ].
			socketStream := SocketStream on: aSocket.
			messageBytes := socketStream upToToken: self class messageEndMark.
			messageBytes ifNotEmpty: [
				message := ReferenceStream unStream: messageBytes.
				(self canProcess: message) 
					ifTrue: [ message sendTo: self recipient ] ] ].
	! !

!Actor methodsFor: 'accessing' stamp: 'mt 9/8/2011 10:30:25.293'!
recipient

	^ recipient ifNil: [self]! !

!Actor methodsFor: 'accessing' stamp: 'mt 9/8/2011 10:30:40.227'!
recipient: anObject

	recipient := anObject.! !

!Actor methodsFor: 'processing' stamp: 'bs 2/21/2010 21:22:28.952'!
runLoop
	"Run the server's connection accept loop."
	| newConnectionSocket |
	
	[listenSocket isValid]
		whileTrue: 
			[[newConnectionSocket := listenSocket waitForAcceptFor: 10]
				on: ConnectionTimedOut
				do: [:ex | newConnectionSocket := nil].
			 newConnectionSocket isNil
				ifFalse: [ self serve: newConnectionSocket ] ]! !

!Actor methodsFor: 'processing' stamp: 'bs 2/25/2010 12:30:57.47'!
serve: aSocket
	[self process: aSocket]
		forkAt: Processor lowIOPriority
	! !

!Actor methodsFor: 'accessing' stamp: 'bs 2/21/2010 20:12:45.289'!
servicePriority
	^ Processor lowIOPriority! !

!Actor methodsFor: 'start/stop' stamp: 'bs 2/21/2010 20:12:30.526'!
startOn: portNumber
	
	listenSocket := Socket newTCP.
	listenSocket listenOn: portNumber backlogSize: 2.
	listenSocket isValid ifFalse: [^self error: 'Could not create server socket'].
	servingProcess := [self runLoop] forkAt: self servicePriority.! !

!Actor methodsFor: 'start/stop' stamp: 'bs 2/21/2010 20:13:14.621'!
stop
	servingProcess terminate.
	listenSocket closeAndDestroy.! !

Actor subclass: #ActorMock
	instanceVariableNames: 'receivedMessage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actors-Tests'!

!ActorMock methodsFor: 'accessing' stamp: 'bs 2/22/2010 08:11:13.701'!
receivedMessage
	^ receivedMessage! !

!ActorMock methodsFor: 'accessing' stamp: 'bs 2/22/2010 08:11:13.703'!
receivedMessage: anObject
	receivedMessage := anObject! !

!ActorMock methodsFor: 'as yet unclassified' stamp: 'bs 2/22/2010 08:12:25.272'!
remember: aMessage
	self receivedMessage: aMessage.! !

Object subclass: #ActorId
	instanceVariableNames: 'hostname port socketStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actors'!

!ActorId class methodsFor: 'instance creation' stamp: 'mt 9/8/2011 09:32:26.059'!
hostname: aName port: anInteger

	^ self new
		hostname: aName
		port: anInteger! !

!ActorId class methodsFor: 'instance creation' stamp: 'mt 9/8/2011 09:32:01.396'!
localhostPort: anInteger
	
	^ self
		hostname: '127.0.0.1'
		port: anInteger! !

!ActorId methodsFor: 'open/close' stamp: 'mt 9/8/2011 09:44:05.286'!
close

	self socketStream close.
	socketStream := nil.! !

!ActorId methodsFor: 'processing' stamp: 'mt 9/8/2011 09:25:56.27'!
doesNotUnderstand: aMessage
	[ self socketStream 
		nextPutAll: (ReferenceStream streamedRepresentationOf: aMessage);
		nextPutAll: Actor messageEndMark;
		flush ] 
			forkAt: Processor lowIOPriority! !

!ActorId methodsFor: 'accessing' stamp: 'mt 9/8/2011 09:44:43.517'!
hostname

	^ hostname! !

!ActorId methodsFor: 'accessing' stamp: 'bs 2/21/2010 21:35:11.327'!
hostname: aName port: anInteger
	hostname := aName.
	port := anInteger.! !

!ActorId methodsFor: 'testing' stamp: 'mt 9/8/2011 09:48:28.705'!
isConnected

	^ socketStream notNil and: [self socketStream isConnected]! !

!ActorId methodsFor: 'open/close' stamp: 'mt 9/8/2011 09:46:02.068'!
open

	| socket |
	self isConnected ifTrue: [self close].	
	
	socket := Socket newTCP.
	socket
		connectTo: (NetNameResolver addressFromString: '127.0.0.1') 
		port: self port.
		
	^ socketStream := SocketStream on: socket! !

!ActorId methodsFor: 'accessing' stamp: 'mt 9/8/2011 09:44:36.927'!
port

	^ port! !

!ActorId methodsFor: 'accessing' stamp: 'mt 9/8/2011 09:44:28.449'!
socketStream

	^ socketStream ifNil: [self open]! !

Object subclass: #ActorIdMock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actors-Tests'!

!ActorIdMock methodsFor: 'as yet unclassified' stamp: 'bs 8/11/2010 14:08:46.681'!
doesNotUnderstand: aMessage! !
